/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.runtime.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.VariableUpdate;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;

/**
 * @author Pierre Vigneras
 * @date Mar 13, 2008
 */
public class ProcessInstanceImpl extends RuntimeRecordImpl implements ProcessInstance {

  private static final long serialVersionUID = 8366284714927360659L;

  protected Date startedDate;
  protected String startedBy;
  protected ProcessInstanceUUID parentInstanceUUID;
  protected Set<ProcessInstanceUUID> childrenInstanceUUID;
  protected InstanceState instanceState;
  protected Map<String, Object> initialVariableValues;
  protected Date endedDate;
  protected String endedBy;

  protected List<VariableUpdate> variableUpdates = new ArrayList<VariableUpdate>();

  public String toString() {
    return this.getClass().getName()
      + "[uuid: " + getUUID()
      + ", packageDefinitionUUID: " + getPackageDefinitionUUID()
      + ", processDefinitionUUID: " + getProcessDefinitionUUID()
      + ", processUUID: " + getProcessInstanceUUID()
      + ", parentInstanceUUID: " + getParentInstanceUUID()
      + ", childrenInstanceUUID: " + getChildrenInstanceUUID()
      + ", startedBy: " + getStartedBy()
      + ", endedBy: " + getEndedBy()
      + ", startedDate: " + getStartedDate()
      + ", endedDate: " + getEndedDate()
      + ", initialVariableValues: " + getInitialVariableValues()
      + "]";
  }

  protected ProcessInstanceImpl() {
    super();
  }

  public ProcessInstanceImpl(final ProcessInstanceUUID instanceUUID,
      final PackageDefinitionUUID packageUUID, final ProcessDefinitionUUID processUUID,
      final ProcessInstanceUUID parentInstanceUUID, 
      final Map<String, Object> initialVariableValues) {
    super(packageUUID, processUUID, instanceUUID);
    this.parentInstanceUUID = parentInstanceUUID;
    this.initialVariableValues = initialVariableValues;
  }

  public ProcessInstanceImpl(final ProcessInstance processInstance) {
    super(processInstance);
    this.startedDate = processInstance.getStartedDate();
    this.startedBy = processInstance.getStartedBy();
    this.parentInstanceUUID = processInstance.getParentInstanceUUID();
    
    Set<ProcessInstanceUUID> children = processInstance.getChildrenInstanceUUID();
    if (children != null && !children.isEmpty()) {
      this.childrenInstanceUUID = new HashSet<ProcessInstanceUUID>();
      for (ProcessInstanceUUID child : children) {
        this.childrenInstanceUUID.add(child);
      }
    }
    
    this.instanceState = processInstance.getInstanceState();
    this.initialVariableValues = processInstance.getInitialVariableValues();
    this.endedDate = processInstance.getEndedDate();
    this.endedBy = processInstance.getEndedBy();
    
    List<VariableUpdate> list = processInstance.getVariableUpdates();
    if (list != null && !list.isEmpty()) {
      this.variableUpdates = new ArrayList<VariableUpdate>();
      for (VariableUpdate varUpdate : list) {
        this.variableUpdates.add(new VariableUpdateImpl(varUpdate));
      }
    }
  }

  public String getEndedBy() {
    return endedBy;
  }

  public Date getEndedDate() {
    return endedDate;
  }

  public Map<String, Object> getInitialVariableValues() {
    return initialVariableValues;
  }

  public InstanceState getInstanceState() {
    return instanceState;
  }

  public ProcessInstanceUUID getParentInstanceUUID() {
    return parentInstanceUUID;
  }

  public Set<ProcessInstanceUUID> getChildrenInstanceUUID() {
    if (this.childrenInstanceUUID == null) {
      return new HashSet<ProcessInstanceUUID>();
    }
    return this.childrenInstanceUUID;
  }
  
  public String getStartedBy() {
    return startedBy;
  }

  public Date getStartedDate() {
    return startedDate;
  }

  public Object getInitialVariableValue(String variableId) {
    return getInitialVariableValues().get(variableId);
  }
  
  public Map<String, Object> getLastKnownVariableValues() {
    Map<String, Object> var = getInitialVariableValues();
    if (var != null) {
      var = new HashMap<String, Object>(var);
    } else {
      var = new HashMap<String, Object>();
    }
    for (VariableUpdate varUp : getVariableUpdates()) {
      var.put(varUp.getName(), varUp.getValue());
    }
    return var;
  }


  public ProcessInstanceUUID getUUID() {
    return getProcessInstanceUUID();
  }

  public List<VariableUpdate> getVariableUpdates() {
    return variableUpdates;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj.getClass().equals(this.getClass()))) {
      return false;
    }
    ProcessInstanceImpl other = (ProcessInstanceImpl)obj;
    if (other.getUUID() == null) {
      return getUUID() == null;
    }
    return other.getUUID().equals(getUUID());
  }

  @Override
  public int hashCode() {
    return getUUID().hashCode();
  }
}
